from concurrent.futures import CancelledError as CancelledError

class RetriesExceededError(Exception):
    last_exception: BaseException
    def __init__(self, last_exception: BaseException, msg: str = ...) -> None: ...

class S3UploadFailedError(Exception): ...
class InvalidSubscriberMethodError(Exception): ...
class TransferNotDoneError(Exception): ...
class FatalError(CancelledError): ...
